"""
CURP Suite
Jacob Sánchez Pérez
Copyright (C) 2021
"""

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

__all__ = [
    "__title__",
    "__summary__",
    "__uri__",
    "__version__",
    "__author__",
    "__email__",
    "__license__",
    "__copyright__",
]

__title__ = "CURPSuite"
__summary__ = "Análisis y validación de la CURP Mexicana"
__uri__ = "https://github.com/jacobszpz/CURPSuite"

__version__ = "2.3.0"

__author__ = "Jacob Sánchez"
__email__ = "jacobszpz@protonmail.com"

__license__ = "GNU General Public License v2"
__copyright__ = f"2021, {__author__}"
